@@ -2,7 +2,8 @@
  * AT-SPI - Assistive Technology Service Provider Interface
  * (Gnome Accessibility Project; http://developer.gnome.org/projects/gap)
  *
- * Copyright 2001 Sun Microsystems Inc.
+ * Copyright 2001, 2002 Sun Microsystems Inc.,
+ * Copyright 2001, 2002 Ximian, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -37,17 +38,29 @@
 #undef SPI_BRIDGE_DEBUG
 
 static CORBA_Environment ev;
-static Accessibility_Registry registry;
+static Accessibility_Registry registry = NULL;
 static SpiApplication *this_app = NULL;
+static gboolean registry_died = FALSE;
 
+static Accessibility_Registry spi_atk_bridge_get_registry (void);
 static void     spi_atk_bridge_exit_func               (void);
 static void     spi_atk_register_event_listeners       (void);
-static gboolean spi_atk_bridge_idle_init               (gpointer               user_data);
 static void     spi_atk_bridge_focus_tracker           (AtkObject             *object);
+static void     spi_atk_bridge_register_application    (Accessibility_Registry registry);
 static gboolean spi_atk_bridge_property_event_listener (GSignalInvocationHint *signal_hint,
 							guint                  n_param_values,
 							const GValue          *param_values,
 							gpointer               data);
+static gboolean
+spi_atk_bridge_window_event_listener (GSignalInvocationHint *signal_hint,
+				guint n_param_values,
+				const GValue *param_values,
+				gpointer data);
+static gboolean
+spi_atk_bridge_state_event_listener (GSignalInvocationHint *signal_hint,
+				     guint n_param_values,
+				     const GValue *param_values,
+				     gpointer data);
 static gboolean spi_atk_bridge_signal_listener         (GSignalInvocationHint *signal_hint,
 							guint                  n_param_values,
 							const GValue          *param_values,
@@ -62,7 +75,6 @@ extern void gnome_accessibility_module_shutdown (void);
 static int     atk_bridge_initialized = FALSE;
 static guint   atk_bridge_focus_tracker_id = 0;
 static guint   atk_bridge_key_event_listener_id = 0;
-static guint   idle_init_id = 0;
 static GArray *listener_ids = NULL;
 
 /*
@@ -99,18 +111,7 @@ atk_bridge_init (gint *argc, gchar **argv[])
 
   CORBA_exception_init(&ev);
 
-  registry = bonobo_activation_activate_from_id (
-	  "OAFIID:Accessibility_Registry:proto0.1", 0, NULL, &ev);
-  
-  if (ev._major != CORBA_NO_EXCEPTION)
-    {
-      g_error ("Accessibility app error: exception during "
-	       "registry activation from id: %s\n",
-	       CORBA_exception_id (&ev));
-      CORBA_exception_free (&ev);
-    }
-
-  if (registry == CORBA_OBJECT_NIL)
+  if (spi_atk_bridge_get_registry () == CORBA_OBJECT_NIL)
     {
       g_error ("Could not locate registry");
     }
@@ -123,35 +124,57 @@ atk_bridge_init (gint *argc, gchar **argv[])
 
   fprintf (stderr, "About to register application\n");
 
-  Accessibility_Registry_registerApplication (registry,
-                                              BONOBO_OBJREF (this_app),
-                                              &ev);
-
+  spi_atk_bridge_register_application (spi_atk_bridge_get_registry ());
+  
   g_atexit (spi_atk_bridge_exit_func);
 
-  idle_init_id = g_idle_add (spi_atk_bridge_idle_init, NULL);
+  fprintf (stderr, "Application registered & listening\n");
 
   return 0;
 }
 
+static void
+spi_atk_bridge_register_application (Accessibility_Registry registry)
+{
+  Accessibility_Registry_registerApplication (spi_atk_bridge_get_registry (),
+                                              BONOBO_OBJREF (this_app),
+                                              &ev);
+  spi_atk_register_event_listeners ();
+}
+
+static Accessibility_Registry
+spi_atk_bridge_get_registry ()
+{
+  CORBA_Environment ev;
+
+  if (registry_died || (registry == NULL)) {
+	  CORBA_exception_init (&ev);
+	  if (registry_died) g_warning ("registry died! restarting...");
+	  registry = bonobo_activation_activate_from_id (
+		  "OAFIID:Accessibility_Registry:1.0", 0, NULL, &ev);
+	  
+	  if (ev._major != CORBA_NO_EXCEPTION)
+	  {
+		  g_error ("Accessibility app error: exception during "
+			   "registry activation from id: %s\n",
+			   CORBA_exception_id (&ev));
+		  CORBA_exception_free (&ev);
+	  }
+	  
+	  if (registry_died && registry) {
+		  registry_died = FALSE;
+		  spi_atk_bridge_register_application (registry);
+	  }
+  }
+  return registry;
+}
+
 int
 gtk_module_init (gint *argc, gchar **argv[])
 {
 	return atk_bridge_init (argc, argv);
 }
 
-static gboolean
-spi_atk_bridge_idle_init (gpointer user_data)
-{
-  idle_init_id = 0;
-
-  spi_atk_register_event_listeners ();
-
-  fprintf (stderr, "Application registered & listening\n");
-
-  return FALSE;
-}
-
 static void
 add_signal_listener (const char *signal_name)
 {
@@ -183,6 +206,30 @@ spi_atk_register_event_listeners (void)
   id = atk_add_global_event_listener (spi_atk_bridge_property_event_listener,
 				      "Gtk:AtkObject:property-change");
   g_array_append_val (listener_ids, id);
+  id = atk_add_global_event_listener (spi_atk_bridge_window_event_listener,
+				      "window:create");
+  g_array_append_val (listener_ids, id);
+  id = atk_add_global_event_listener (spi_atk_bridge_window_event_listener,
+				      "window:destroy");
+  g_array_append_val (listener_ids, id);
+  id = atk_add_global_event_listener (spi_atk_bridge_window_event_listener,
+				      "window:minimize");
+  g_array_append_val (listener_ids, id);
+  id = atk_add_global_event_listener (spi_atk_bridge_window_event_listener,
+				      "window:maximize");
+  g_array_append_val (listener_ids, id);
+  id = atk_add_global_event_listener (spi_atk_bridge_window_event_listener,
+				      "window:restore");
+  g_array_append_val (listener_ids, id);
+  id = atk_add_global_event_listener (spi_atk_bridge_window_event_listener,
+				      "window:activate");
+  g_array_append_val (listener_ids, id);
+  id = atk_add_global_event_listener (spi_atk_bridge_window_event_listener,
+				      "window:deactivate");
+  g_array_append_val (listener_ids, id);
+  id = atk_add_global_event_listener (spi_atk_bridge_state_event_listener,
+				      "Gtk:AtkObject:state-change");
+  g_array_append_val (listener_ids, id);
 
   add_signal_listener ("Gtk:AtkObject:children-changed");
   add_signal_listener ("Gtk:AtkObject:visible-data-changed");
@@ -213,8 +260,8 @@ spi_atk_register_event_listeners (void)
 static void
 deregister_application (BonoboObject *app)
 {
-  Accessibility_Registry_deregisterApplication (
-	  registry, BONOBO_OBJREF (app), &ev);
+  Accessibility_Registry registry = spi_atk_bridge_get_registry ();	
+  Accessibility_Registry_deregisterApplication (registry, BONOBO_OBJREF (app), &ev);
 
   registry = bonobo_object_release_unref (registry, &ev);
   
@@ -268,7 +315,9 @@ void
 gnome_accessibility_module_shutdown (void)
 {
   BonoboObject *app = (BonoboObject *) this_app;
-
+  int     i;
+  GArray *ids = listener_ids;
+  
   if (!atk_bridge_initialized)
     {
       return;
@@ -278,26 +327,15 @@ gnome_accessibility_module_shutdown (void)
 
   g_print("Atk Accessibilty bridge shutdown\n");
 
-  if (idle_init_id)
-    {
-      g_source_remove (idle_init_id);
-      idle_init_id = 0;
-    }
-  else
-    {
-      int     i;
-      GArray *ids = listener_ids;
-
-      listener_ids = NULL;
-      atk_remove_focus_tracker (atk_bridge_focus_tracker_id);
-      
-      for (i = 0; ids && i < ids->len; i++)
-        {
+  listener_ids = NULL;
+  atk_remove_focus_tracker (atk_bridge_focus_tracker_id);
+  
+  for (i = 0; ids && i < ids->len; i++)
+  {
           atk_remove_global_event_listener (g_array_index (ids, guint, i));
-	}
-
-      atk_remove_key_event_listener (atk_bridge_key_event_listener_id);
-    }
+  }
+  
+  atk_remove_key_event_listener (atk_bridge_key_event_listener_id);
 
   deregister_application (app);
 }
@@ -315,8 +353,11 @@ spi_atk_bridge_focus_tracker (AtkObject *object)
   e.detail1 = 0;
   e.detail2 = 0;
 
-  Accessibility_Registry_notifyEvent (registry, &e, &ev);
-
+  Accessibility_Registry_notifyEvent (spi_atk_bridge_get_registry (), &e, &ev);
+  if (BONOBO_EX (&ev)) registry_died = TRUE;
+  
+  Accessibility_Accessible_unref (e.source, &ev);
+  
   CORBA_exception_free (&ev);
 }
 
@@ -368,7 +409,16 @@ spi_atk_emit_eventv (GObject      *gobject,
       CORBA_free (s);
 #endif
 
-      Accessibility_Registry_notifyEvent (registry, &e, &ev);
+      Accessibility_Registry_notifyEvent (spi_atk_bridge_get_registry (), &e, &ev);
+#ifdef SPI_BRIDGE_DEBUG
+      if (ev._major != CORBA_NO_EXCEPTION)
+	      g_warning ("error emitting event %s, (%d) %s",
+			 e.type,
+			 ev._major,
+			 CORBA_exception_id(&ev));
+#endif	      
+      if (BONOBO_EX (&ev)) registry_died = TRUE;
+      Accessibility_Accessible_unref (e.source, &ev);
 
       CORBA_exception_free (&ev);
 
@@ -376,6 +426,7 @@ spi_atk_emit_eventv (GObject      *gobject,
     }
 
   va_end (args);
+
 }
 
 static gboolean
@@ -390,7 +441,7 @@ spi_atk_bridge_property_event_listener (GSignalInvocationHint *signal_hint,
 #ifdef SPI_BRIDGE_DEBUG
   GSignalQuery signal_query;
   const gchar *name;
-  gchar *s, *s2;
+  const gchar *s, *s2;
   
   g_signal_query (signal_hint->signal_id, &signal_query);
   name = signal_query.signal_name;
@@ -411,7 +462,6 @@ spi_atk_bridge_property_event_listener (GSignalInvocationHint *signal_hint,
   return TRUE;
 }
 
-#if THIS_WILL_EVER_BE_USED
 static gboolean
 spi_atk_bridge_state_event_listener (GSignalInvocationHint *signal_hint,
 				     guint n_param_values,
@@ -419,7 +469,9 @@ spi_atk_bridge_state_event_listener (GSignalInvocationHint *signal_hint,
 				     gpointer data)
 {
   GObject *gobject;
-  AtkPropertyValues *values;
+  gchar *property_name;
+  gchar *type;
+  unsigned long detail1;
 #ifdef SPI_BRIDGE_DEBUG
   GSignalQuery signal_query;
   const gchar *name;
@@ -431,16 +483,19 @@ spi_atk_bridge_state_event_listener (GSignalInvocationHint *signal_hint,
 #endif
 
   gobject = g_value_get_object (param_values + 0);
-  values = (AtkPropertyValues*) g_value_get_pointer (param_values + 1);
-
+  property_name = g_strdup (g_value_get_string (param_values + 1));
+  detail1 = (g_value_get_boolean (param_values + 2))
+    ? 1 : 0;
+  type = g_strdup_printf ("object:state-changed:%s", property_name);
   spi_atk_emit_eventv (gobject, 
-		       (unsigned long) values->old_value.data[0].v_ulong,
-		       (unsigned long) values->new_value.data[0].v_ulong,
-		       "object:%s:?", values->property_name);
-  
+		       detail1,
+		       0,
+		       type);
+  g_free (property_name);
+  g_free (type);
   return TRUE;
 }
-#endif
+
 
 static void
 spi_init_keystroke_from_atk_key_event (Accessibility_DeviceEvent  *keystroke,
@@ -475,10 +530,10 @@ spi_init_keystroke_from_atk_key_event (Accessibility_DeviceEvent  *keystroke,
   switch (event->type)
     {
     case (ATK_KEY_EVENT_PRESS):
-      keystroke->type = Accessibility_KEY_PRESSED;
+      keystroke->type = Accessibility_KEY_PRESSED_EVENT;
       break;
     case (ATK_KEY_EVENT_RELEASE):
-      keystroke->type = Accessibility_KEY_RELEASED;
+      keystroke->type = Accessibility_KEY_RELEASED_EVENT;
       break;
     default:
       keystroke->type = 0;
@@ -497,13 +552,20 @@ spi_atk_bridge_key_listener (AtkKeyEventStruct *event, gpointer data)
 {
   CORBA_boolean             result;
   Accessibility_DeviceEvent key_event;
-  Accessibility_DeviceEventController controller =
-    Accessibility_Registry_getDeviceEventController (registry, &ev);
+  Accessibility_DeviceEventController controller;
+	
+  if (BONOBO_EX (&ev))
+	g_warning ("failure: pre-listener get dec\n");
+
+  controller =
+    Accessibility_Registry_getDeviceEventController (
+	    spi_atk_bridge_get_registry (), &ev);
 
   if (BONOBO_EX (&ev))
     {
       g_warning ("failure: no deviceeventcontroller found\n");
       CORBA_exception_free (&ev);
+      registry_died = TRUE;
       result = FALSE;
     }
   else
@@ -514,6 +576,7 @@ spi_atk_bridge_key_listener (AtkKeyEventStruct *event, gpointer data)
       result = Accessibility_DeviceEventController_notifyListenersSync (
         controller, &key_event, &ev);
 
+      bonobo_object_release_unref (controller, &ev);
       CORBA_exception_free (&ev);
     }
 
@@ -529,9 +592,54 @@ spi_atk_bridge_signal_listener (GSignalInvocationHint *signal_hint,
   GObject *gobject;
   GSignalQuery signal_query;
   const gchar *name;
+  const gchar *detail;
+  
   gint detail1 = 0, detail2 = 0;
 #ifdef SPI_BRIDGE_DEBUG
-  gchar *s, *s2;
+  const gchar *s, *s2;
+#endif
+  
+  g_signal_query (signal_hint->signal_id, &signal_query);
+
+  name = signal_query.signal_name;
+  if (signal_hint->detail)
+    detail = g_quark_to_string (signal_hint->detail);
+  else
+    detail = NULL;
+
+#ifdef SPI_BRIDGE_DEBUG
+  s2 = g_type_name (G_OBJECT_TYPE (g_value_get_object (param_values + 0)));
+  s = atk_object_get_name (ATK_OBJECT (g_value_get_object (param_values + 0)));
+  fprintf (stderr, "Received signal %s:%s detail: %s from object %s (gail %s)\n",
+	   g_type_name (signal_query.itype), name, 
+                        detail ? detail : "<NULL>", s ? s : "<NULL>" , s2);
+#endif
+
+  gobject = g_value_get_object (param_values + 0);
+  if (G_VALUE_TYPE (param_values + 1) == G_TYPE_INT)
+    detail1 = g_value_get_int (param_values + 1);
+  if (G_VALUE_TYPE (param_values + 2) == G_TYPE_INT)
+    detail2 = g_value_get_int (param_values + 2);
+  
+  if (detail)
+    spi_atk_emit_eventv (gobject, detail1, detail2, "object:%s:%s", name, detail);
+  else
+    spi_atk_emit_eventv (gobject, detail1, detail2, "object:%s", name);
+
+  return TRUE;
+}
+
+static gboolean
+spi_atk_bridge_window_event_listener (GSignalInvocationHint *signal_hint,
+				guint n_param_values,
+				const GValue *param_values,
+				gpointer data)
+{
+  GObject *gobject;
+  GSignalQuery signal_query;
+  const gchar *name;
+#ifdef SPI_BRIDGE_DEBUG
+  const gchar *s, *s2;
 #endif
   
   g_signal_query (signal_hint->signal_id, &signal_query);
@@ -542,16 +650,11 @@ spi_atk_bridge_signal_listener (GSignalInvocationHint *signal_hint,
   s2 = g_type_name (G_OBJECT_TYPE (g_value_get_object (param_values + 0)));
   s = atk_object_get_name (ATK_OBJECT (g_value_get_object (param_values + 0)));
   fprintf (stderr, "Received signal %s:%s from object %s (gail %s)\n",
-	   g_type_name (signal_query.itype), name, s, s2);
+	   g_type_name (signal_query.itype), name, s ? s : "<NULL>" , s2);
 #endif
 
   gobject = g_value_get_object (param_values + 0);
-  if (G_VALUE_TYPE (param_values + 1) == G_TYPE_INT)
-    detail1 = g_value_get_int (param_values + 1);
-  if (G_VALUE_TYPE (param_values + 2) == G_TYPE_INT)
-    detail2 = g_value_get_int (param_values + 2);
-  
-  spi_atk_emit_eventv (gobject, detail1, detail2, "object:%s", name);
+  spi_atk_emit_eventv (gobject, 0, 0, "window:%s", name);
 
   return TRUE;
 }
