@@ -2,7 +2,8 @@
  * AT-SPI - Assistive Technology Service Provider Interface
  * (Gnome Accessibility Project; http://developer.gnome.org/projects/gap)
  *
- * Copyright 2001 Sun Microsystems Inc.
+ * Copyright 2001, 2002 Sun Microsystems Inc.,
+ * Copyright 2001, 2002 Ximian, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -161,19 +162,13 @@ spi_dec_poll_mouse_moved (gpointer data)
   unsigned int mask_return;
   gchar event_name[24];
   Display *display = spi_get_display ();
-  if (last_mouse_pos == NULL) {
-	  last_mouse_pos = g_new0 (GdkPoint, 1);
-	  last_mouse_pos->x = 0;
-	  last_mouse_pos->y = 0;
-	  e.type = g_strdup ("mouse:abs");
-  } else {
-	  e.type = g_strdup ("mouse:rel");
-  }
+
   if (display != NULL)
 	  XQueryPointer(display, DefaultRootWindow (display),
 		&root_return, &child_return,
 		&x, &y,
 		&win_x_return, &win_y_return, &mask_return);
+
   if ((mask_return & mouse_button_mask) != mouse_button_state) {
 	  int button_number = 0;
 	  if (!(mask_return & Button1Mask) &&
@@ -198,7 +193,7 @@ spi_dec_poll_mouse_moved (gpointer data)
 			   button_number);
 #endif
 		  snprintf (event_name, 22, "mouse:button:%dr", button_number);
-		  e.type = CORBA_string_dup (event_name);
+		  e.type = event_name;
 		  e.source = BONOBO_OBJREF (registry->desktop);
 		  e.detail1 = last_mouse_pos->x;
 		  e.detail2 = last_mouse_pos->y;
@@ -209,13 +204,19 @@ spi_dec_poll_mouse_moved (gpointer data)
 	  }
 	  mouse_button_state = mask_return & mouse_button_mask;
   }
+  if (last_mouse_pos == NULL) {
+	  last_mouse_pos = g_new0 (GdkPoint, 1);
+	  last_mouse_pos->x = 0;
+	  last_mouse_pos->y = 0;
+	  e.type = "mouse:abs";
+  } else {
+	  e.type = "mouse:rel";  
+  }
   if (x != last_mouse_pos->x || y != last_mouse_pos->y) {
 	  e.source = BONOBO_OBJREF (registry->desktop);
 	  e.detail1 = x - last_mouse_pos->x;
 	  e.detail2 = y - last_mouse_pos->y;
 	  CORBA_exception_init (&ev);
-	  if (last_mouse_pos == NULL)
-		  last_mouse_pos = g_new0 (GdkPoint, 1);
 	  last_mouse_pos->x = x;
 	  last_mouse_pos->y = y;
 	  Accessibility_Registry_notifyEvent (BONOBO_OBJREF (registry),
@@ -415,7 +416,7 @@ handle_keygrab (SpiDEController         *controller,
     {
       grab_mask.key_val = AnyKey;
 #ifdef SPI_DEBUG
-      fprintf (stderr, "AnyKey grab!"); */
+      fprintf (stderr, "AnyKey grab!");
 #endif
       process_cb (controller, &grab_mask);
     }
@@ -547,7 +548,7 @@ global_filter_fn (GdkXEvent *gdk_xevent, GdkEvent *event, gpointer data)
   XEvent *xevent = gdk_xevent;
   SpiDEController *controller;
 
-  if (xevent->type == KeyPress && xevent->type == KeyRelease)
+  if (xevent->type == KeyPress || xevent->type == KeyRelease)
     {
       controller = SPI_DEVICE_EVENT_CONTROLLER (data);
       spi_device_event_controller_forward_key_event (controller, xevent);
@@ -593,7 +594,6 @@ spi_controller_register_with_devices (SpiDEController *controller)
   x_default_error_handler = XSetErrorHandler (_spi_controller_device_error_handler);
 }
 
-#define SPI_KEYEVENT_DEBUG
 static gboolean
 spi_key_set_contains_key (Accessibility_KeySet            *key_set,
 			  const Accessibility_DeviceEvent *key_event)
@@ -972,15 +972,15 @@ spi_controller_update_key_grabs (SpiDEController           *controller,
         {
 
 #ifdef SPI_DEBUG
-	  fprintf (stderr, "grab with mask %x\n", grab_mask->mod_mask);
+	  fprintf (stderr, "grab %d with mask %x\n", grab_mask->key_val, grab_mask->mod_mask);
 #endif
           XGrabKey (spi_get_display (),
 		    grab_mask->key_val,
 		    grab_mask->mod_mask,
 		    gdk_x11_get_default_root_xwindow (),
 		    True,
-		    GrabModeAsync,
-		    GrabModeAsync);
+		    GrabModeSync,
+		    GrabModeSync);
 	  XSync (spi_get_display (), False);
 	  update_failed = spi_clear_error_state ();
 	  if (update_failed) {
